@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -42,12 +42,11 @@ u32 mul_hash_64(u64a lv, u64a andmsk, u64a mult, u32 nBits) {
 #define CONF_TYPE u64a
 #define CONF_HASH_CALL mul_hash_64
 
-typedef enum LitInfoFlags {
-    NoFlags = 0,
-    Caseless = 1,
-    NoRepeat = 2,
-    ComplexConfirm = 4
-} LitInfoFlags;
+/**
+ * \brief Flag indicating this literal doesn't need to be delivered more than
+ * once, used in LitInfo::flags.
+ */
+#define FDR_LIT_FLAG_NOREPEAT   1
 
 /**
  * \brief Structure describing a literal, linked to by FDRConfirm.
@@ -61,12 +60,12 @@ struct LitInfo {
     hwlm_group_t groups;
     u32 id; // literal ID as passed in
     u8 size;
-    u8 flags; /* LitInfoFlags */
+    u8 flags; //!< bitfield of flags from FDR_LIT_FLAG_* above.
     u8 next;
-    u8 extended_size;
 };
 
 #define FDRC_FLAG_NO_CONFIRM 1
+#define FDRC_FLAG_NOREPEAT   2
 
 /**
  * \brief FDR confirm header.
@@ -79,12 +78,8 @@ struct LitInfo {
 struct FDRConfirm {
     CONF_TYPE andmsk;
     CONF_TYPE mult;
-    u32 nBitsOrSoleID; // if flags is NO_CONFIRM then this is soleID
-    u32 flags;  // sole meaning is 'non-zero means no-confirm' (that is all)
+    u32 nBits;
     hwlm_group_t groups;
-    u32 soleLitSize;
-    u32 soleLitCmp;
-    u32 soleLitMsk;
 };
 
 static really_inline
