@@ -655,12 +655,6 @@ char nfaExecGough8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
     const u8 *cur_buf = sp < 0 ? hend : buffer;
 
-    char report = 1;
-    if (mode == CALLBACK_OUTPUT) {
-        /* we are starting inside the history buffer: matches are suppressed */
-        report = !(sp < 0);
-    }
-
     if (mode != NO_MATCHES && q->items[q->cur - 1].location > end) {
         /* this is as far as we go */
         q->cur--;
@@ -691,8 +685,7 @@ char nfaExecGough8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         const u8 *final_look;
         if (goughExec8_i_ni(m, som, &s, cur_buf + sp, local_ep - sp,
-                            offset + sp, cb, context, &final_look,
-                            report ? mode : NO_MATCHES)
+                            offset + sp, cb, context, &final_look, mode)
             == MO_HALT_MATCHING) {
             *(u8 *)q->state = 0;
             return 0;
@@ -724,7 +717,6 @@ char nfaExecGough8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         if (sp == 0) {
             cur_buf = buffer;
-            report = 1;
         }
 
         if (sp != ep) {
@@ -789,12 +781,6 @@ char nfaExecGough16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
     const u8 *cur_buf = sp < 0 ? hend : buffer;
 
-    char report = 1;
-    if (mode == CALLBACK_OUTPUT) {
-        /* we are starting inside the history buffer: matches are suppressed */
-        report = !(sp < 0);
-    }
-
     assert(q->cur);
     if (mode != NO_MATCHES && q->items[q->cur - 1].location > end) {
         /* this is as far as we go */
@@ -822,10 +808,8 @@ char nfaExecGough16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         /* do main buffer region */
         const u8 *final_look;
         if (goughExec16_i_ni(m, som, &s, cur_buf + sp, local_ep - sp,
-                             offset + sp, cb, context, &final_look,
-                             report ? mode : NO_MATCHES)
+                             offset + sp, cb, context, &final_look, mode)
             == MO_HALT_MATCHING) {
-            assert(report);
             *(u16 *)q->state = 0;
             return 0;
         }
@@ -856,7 +840,6 @@ char nfaExecGough16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         if (sp == 0) {
             cur_buf = buffer;
-            report = 1;
         }
 
         if (sp != ep) {
