@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -42,10 +42,10 @@
 
 static really_inline
 char doComplexReport(NfaCallback cb, void *ctxt, const struct mcclellan *m,
-                     u16 s, u64a loc, char eod, u16 *const cached_accept_state,
-                     u32 *const cached_accept_id) {
-    DEBUG_PRINTF("reporting state = %hu, loc=%llu, eod %hhu\n",
-                 (u16)(s & STATE_MASK), loc, eod);
+                     u32 s, u64a loc, char eod, u32 *cached_accept_state,
+                     u32 *cached_accept_id) {
+    DEBUG_PRINTF("reporting state = %u, loc=%llu, eod %hhu\n",
+                 s & STATE_MASK, loc, eod);
 
     if (!eod && s == *cached_accept_state) {
         if (cb(0, loc, *cached_accept_id, ctxt) == MO_HALT_MATCHING) {
@@ -89,27 +89,108 @@ char doComplexReport(NfaCallback cb, void *ctxt, const struct mcclellan *m,
 }
 
 static really_inline
-char mcclellanExec16_i(const struct mcclellan *m, u16 *state, const u8 *buf,
+const u8 *run_mcclellan_accel(const struct mcclellan *m,
+                              const struct mstate_aux *aux, u32 s,
+                              const u8 **min_accel_offset,
+                              const u8 *c, const u8 *c_end) {
+    DEBUG_PRINTF("skipping\n");
+    u32 accel_offset = aux[s].accel_offset;
+
+    assert(aux[s].accel_offset);
+    assert(accel_offset >= m->aux_offset);
+    assert(!m->sherman_offset || accel_offset < m->sherman_offset);
+
+    const union AccelAux *aaux = (const void *)((const char *)m + accel_offset);
+    const u8 *c2 = run_accel(aaux, c, c_end);
+
+    if (c2 < *min_accel_offset + BAD_ACCEL_DIST) {
+        *min_accel_offset = c2 + BIG_ACCEL_PENALTY;
+    } else {
+        *min_accel_offset = c2 + SMALL_ACCEL_PENALTY;
+    }
+
+    if (*min_accel_offset >= c_end - ACCEL_MIN_LEN) {
+        *min_accel_offset = c_end;
+    }
+
+    DEBUG_PRINTF("advanced %zd, next accel chance in %zd/%zd\n",
+                 c2 - c, *min_accel_offset - c2, c_end - c2);
+
+    return c2;
+}
+
+static really_inline
+u32 doNormal16(const struct mcclellan *m, const u8 **c_inout, const u8 *end,
+               u32 s, char do_accel, enum MatchMode mode) {
+    const u8 *c = *c_inout;
+
+    const u16 *succ_table
+        = (const u16 *)((const char *)m + sizeof(struct mcclellan));
+    assert(ISALIGNED_N(succ_table, 2));
+    u32 sherman_base = m->sherman_limit;
+    const char *sherman_base_offset
+        = (const char *)m - sizeof(struct NFA) + m->sherman_offset;
+    u32 as = m->alphaShift;
+
+    s &= STATE_MASK;
+
+    while (c < end && s) {
+        u8 cprime = m->remap[*c];
+        DEBUG_PRINTF("c: %02hhx '%c' cp:%02hhx (s=%u)\n", *c,
+                     ourisprint(*c) ? *c : '?', cprime, s);
+        if (s < sherman_base) {
+            DEBUG_PRINTF("doing normal\n");
+            assert(s < m->state_count);
+            s = succ_table[(s << as) + cprime];
+        } else {
+            const char *sherman_state
+                = findShermanState(m, sherman_base_offset, sherman_base, s);
+            DEBUG_PRINTF("doing sherman (%u)\n", s);
+            s = doSherman16(sherman_state, cprime, succ_table, as);
+        }
+
+        DEBUG_PRINTF("s: %u (%u)\n", s, s & STATE_MASK);
+        c++;
+
+        if (do_accel && (s & ACCEL_FLAG)) {
+            break;
+        }
+        if (mode != NO_MATCHES && (s & ACCEPT_FLAG)) {
+            break;
+        }
+
+        s &= STATE_MASK;
+    }
+
+    *c_inout = c;
+    return s;
+}
+
+static really_inline
+char mcclellanExec16_i(const struct mcclellan *m, u32 *state, const u8 *buf,
                        size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                        char single, const u8 **c_final, enum MatchMode mode) {
     assert(ISALIGNED_N(state, 2));
+    if (!len) {
+        if (mode == STOP_AT_MATCH) {
+            *c_final = buf;
+        }
+        return MO_ALIVE;
+    }
 
-    u16 s = *state;
-    const u8 *c = buf, *c_end = buf + len;
-    const u16 *succ_table = (const u16 *)((const char *)m
-                                          + sizeof(struct mcclellan));
-    assert(ISALIGNED_N(succ_table, 2));
-    const u16 sherman_base = m->sherman_limit;
-    const char *sherman_base_offset
-        = (const char *)m - sizeof(struct NFA) + m->sherman_offset;
-    const u32 as = m->alphaShift;
+    u32 s = *state;
+    const u8 *c = buf;
+    const u8 *c_end = buf + len;
+    const struct mstate_aux *aux
+        = (const struct mstate_aux *)((const char *)m + m->aux_offset
+                                      - sizeof(struct NFA));
 
     s &= STATE_MASK;
 
     u32 cached_accept_id = 0;
-    u16 cached_accept_state = 0;
+    u32 cached_accept_state = 0;
 
-    DEBUG_PRINTF("s: %hu, len %zu\n", s, len);
+    DEBUG_PRINTF("s: %u, len %zu\n", s, len);
 
     const u8 *min_accel_offset = c;
     if (!m->has_accel || len < ACCEL_MIN_LEN) {
@@ -120,26 +201,19 @@ char mcclellanExec16_i(const struct mcclellan *m, u16 *state, const u8 *buf,
     goto with_accel;
 
 without_accel:
-    while (c < min_accel_offset && s) {
-        u8 cprime = m->remap[*(c++)];
-        DEBUG_PRINTF("c: %02hhx cp:%02hhx (s=%hu)\n", *(c-1), cprime, s);
-        if (s < sherman_base) {
-            DEBUG_PRINTF("doing normal\n");
-            assert(s < m->state_count);
-            s = succ_table[((u32)s << as) + cprime];
-        } else {
-            const char *sherman_state
-                = findShermanState(m, sherman_base_offset, sherman_base, s);
-            DEBUG_PRINTF("doing sherman (%hu)\n", s);
-            s = doSherman16(sherman_state, cprime, succ_table, as);
+    do {
+        assert(c < min_accel_offset);
+        if (!s) {
+            goto exit;
         }
-        DEBUG_PRINTF("s: %hu (%hu)\n", s, (u16)(s & STATE_MASK));
+
+        s = doNormal16(m, &c, min_accel_offset, s, 0, mode);
 
         if (mode != NO_MATCHES && (s & ACCEPT_FLAG)) {
             if (mode == STOP_AT_MATCH) {
                 *state = s & STATE_MASK;
                 *c_final = c - 1;
-                return MO_CONTINUE_MATCHING;
+                return MO_MATCHES_PENDING;
             }
 
             u64a loc = (c - 1) - buf + offAdj + 1;
@@ -147,39 +221,51 @@ without_accel:
             if (single) {
                 DEBUG_PRINTF("reporting %u\n", m->arb_report);
                 if (cb(0, loc, m->arb_report, ctxt) == MO_HALT_MATCHING) {
-                    return MO_HALT_MATCHING; /* termination requested */
+                    return MO_DEAD; /* termination requested */
                 }
             } else if (doComplexReport(cb, ctxt, m, s & STATE_MASK, loc, 0,
-                                       &cached_accept_state,
-                                       &cached_accept_id) == MO_HALT_MATCHING) {
-                return MO_HALT_MATCHING;
+                                       &cached_accept_state, &cached_accept_id)
+                       == MO_HALT_MATCHING) {
+                return MO_DEAD;
             }
         }
 
-        s &= STATE_MASK;
+        assert(c <= min_accel_offset);
+    } while (c < min_accel_offset);
+
+    s &= STATE_MASK;
+
+    if (c == c_end) {
+        goto exit;
+    } else {
+        goto with_accel;
     }
 
 with_accel:
-    while (c < c_end && s) {
-        u8 cprime = m->remap[*(c++)];
-        DEBUG_PRINTF("c: %02hhx cp:%02hhx (s=%hu)\n", *(c-1), cprime, s);
-        if (s < sherman_base) {
-            DEBUG_PRINTF("doing normal\n");
-            assert(s < m->state_count);
-            s = succ_table[((u32)s << as) + cprime];
-        } else {
-            const char *sherman_state
-                = findShermanState(m, sherman_base_offset, sherman_base, s);
-            DEBUG_PRINTF("doing sherman (%hu)\n", s);
-            s = doSherman16(sherman_state, cprime, succ_table, as);
+    do {
+        assert(c < c_end);
+        if (!s) {
+            goto exit;
         }
-        DEBUG_PRINTF("s: %hu (%hu)\n", s, (u16)(s & STATE_MASK));
+
+        if (s & ACCEL_FLAG) {
+            DEBUG_PRINTF("skipping\n");
+            s &= STATE_MASK;
+            c = run_mcclellan_accel(m, aux, s, &min_accel_offset, c, c_end);
+            if (c == c_end) {
+                goto exit;
+            } else {
+                goto without_accel;
+            }
+        }
+
+        s = doNormal16(m, &c, c_end, s, 1, mode);
 
         if (mode != NO_MATCHES && (s & ACCEPT_FLAG)) {
             if (mode == STOP_AT_MATCH) {
                 *state = s & STATE_MASK;
                 *c_final = c - 1;
-                return MO_CONTINUE_MATCHING;
+                return MO_MATCHES_PENDING;
             }
 
             u64a loc = (c - 1) - buf + offAdj + 1;
@@ -187,56 +273,31 @@ with_accel:
             if (single) {
                 DEBUG_PRINTF("reporting %u\n", m->arb_report);
                 if (cb(0, loc, m->arb_report, ctxt) == MO_HALT_MATCHING) {
-                    return MO_HALT_MATCHING; /* termination requested */
+                    return MO_DEAD; /* termination requested */
                 }
             } else if (doComplexReport(cb, ctxt, m, s & STATE_MASK, loc, 0,
-                                       &cached_accept_state,
-                                       &cached_accept_id) == MO_HALT_MATCHING) {
-                return MO_HALT_MATCHING;
+                                       &cached_accept_state, &cached_accept_id)
+                       == MO_HALT_MATCHING) {
+                return MO_DEAD;
             }
-        } else if (s & ACCEL_FLAG) {
-            DEBUG_PRINTF("skipping\n");
-            const struct mstate_aux *this_aux = get_aux(m, s & STATE_MASK);
-            u32 accel_offset = this_aux->accel_offset;
-
-            assert(accel_offset >= m->aux_offset);
-            assert(accel_offset < m->sherman_offset);
-
-            const union AccelAux *aaux
-                = (const void *)((const char *)m + accel_offset);
-            const u8 *c2 = run_accel(aaux, c, c_end);
-
-            if (c2 < min_accel_offset + BAD_ACCEL_DIST) {
-                min_accel_offset = c2 + BIG_ACCEL_PENALTY;
-            } else {
-                min_accel_offset = c2 + SMALL_ACCEL_PENALTY;
-            }
-
-            if (min_accel_offset >= c_end - ACCEL_MIN_LEN) {
-                min_accel_offset = c_end;
-            }
-
-            DEBUG_PRINTF("advanced %zd, next accel chance in %zd/%zd\n",
-                         c2 - c, min_accel_offset - c2, c_end - c2);
-
-            c = c2;
-            s &= STATE_MASK;
-            goto without_accel;
         }
 
-        s &= STATE_MASK;
-    }
+        assert(c <= c_end);
+    } while (c < c_end);
+
+exit:
+    s &= STATE_MASK;
 
     if (mode == STOP_AT_MATCH) {
         *c_final = c_end;
     }
     *state = s;
 
-    return MO_CONTINUE_MATCHING;
+    return MO_ALIVE;
 }
 
 static never_inline
-char mcclellanExec16_i_cb(const struct mcclellan *m, u16 *state, const u8 *buf,
+char mcclellanExec16_i_cb(const struct mcclellan *m, u32 *state, const u8 *buf,
                           size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                           char single, const u8 **final_point) {
     return mcclellanExec16_i(m, state, buf, len, offAdj, cb, ctxt, single,
@@ -244,7 +305,7 @@ char mcclellanExec16_i_cb(const struct mcclellan *m, u16 *state, const u8 *buf,
 }
 
 static never_inline
-char mcclellanExec16_i_sam(const struct mcclellan *m, u16 *state, const u8 *buf,
+char mcclellanExec16_i_sam(const struct mcclellan *m, u32 *state, const u8 *buf,
                            size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                            char single, const u8 **final_point) {
     return mcclellanExec16_i(m, state, buf, len, offAdj, cb, ctxt, single,
@@ -252,15 +313,15 @@ char mcclellanExec16_i_sam(const struct mcclellan *m, u16 *state, const u8 *buf,
 }
 
 static never_inline
-char mcclellanExec16_i_nm(const struct mcclellan *m, u16 *state, const u8 *buf,
-                           size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
-                           char single, const u8 **final_point) {
+char mcclellanExec16_i_nm(const struct mcclellan *m, u32 *state, const u8 *buf,
+                          size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
+                          char single, const u8 **final_point) {
     return mcclellanExec16_i(m, state, buf, len, offAdj, cb, ctxt, single,
                              final_point, NO_MATCHES);
 }
 
 static really_inline
-char mcclellanExec16_i_ni(const struct mcclellan *m, u16 *state, const u8 *buf,
+char mcclellanExec16_i_ni(const struct mcclellan *m, u32 *state, const u8 *buf,
                           size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                           char single, const u8 **final_point,
                           enum MatchMode mode) {
@@ -271,35 +332,69 @@ char mcclellanExec16_i_ni(const struct mcclellan *m, u16 *state, const u8 *buf,
         return mcclellanExec16_i_sam(m, state, buf, len, offAdj, cb, ctxt,
                                      single, final_point);
     } else {
-        assert (mode == NO_MATCHES);
+        assert(mode == NO_MATCHES);
         return mcclellanExec16_i_nm(m, state, buf, len, offAdj, cb, ctxt,
                                     single, final_point);
     }
 }
 
 static really_inline
-char mcclellanExec8_i(const struct mcclellan *m, u8 *state, const u8 *buf,
-                      size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
-                      char single, const u8 **c_final, enum MatchMode mode) {
-    u8 s = *state;
-    const u8 *c = buf, *c_end = buf + len;
+u32 doNormal8(const struct mcclellan *m, const u8 **c_inout, const u8 *end,
+              u32 s, char do_accel, enum MatchMode mode) {
+    const u8 *c = *c_inout;
+    u32 accel_limit = m->accel_limit_8;
+    u32 accept_limit = m->accept_limit_8;
+
+    const u32 as = m->alphaShift;
     const u8 *succ_table = (const u8 *)((const char *)m
                                         + sizeof(struct mcclellan));
-    const u32 as = m->alphaShift;
-    const struct mstate_aux *aux;
+    while (c < end && s) {
+        u8 cprime = m->remap[*c];
+        DEBUG_PRINTF("c: %02hhx '%c' cp:%02hhx\n", *c,
+                     ourisprint(*c) ? *c : '?', cprime);
+        s = succ_table[(s << as) + cprime];
 
-    aux = (const struct mstate_aux *)((const char *)m + m->aux_offset
+        DEBUG_PRINTF("s: %u\n", s);
+        c++;
+        if (do_accel) {
+            if (s >= accel_limit) {
+                break;
+            }
+        } else {
+            if (mode != NO_MATCHES && s >= accept_limit) {
+                break;
+            }
+        }
+    }
+    *c_inout = c;
+    return s;
+}
+
+static really_inline
+char mcclellanExec8_i(const struct mcclellan *m, u32 *state, const u8 *buf,
+                      size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
+                      char single, const u8 **c_final, enum MatchMode mode) {
+    if (!len) {
+        if (mode == STOP_AT_MATCH) {
+            *c_final = buf;
+        }
+        return MO_ALIVE;
+    }
+    u32 s = *state;
+    const u8 *c = buf;
+    const u8 *c_end = buf + len;
+
+    const struct mstate_aux *aux
+        = (const struct mstate_aux *)((const char *)m + m->aux_offset
                                       - sizeof(struct NFA));
-
-    u16 accel_limit = m->accel_limit_8;
-    u16 accept_limit = m->accept_limit_8;
+    u32 accept_limit = m->accept_limit_8;
 
     u32 cached_accept_id = 0;
-    u16 cached_accept_state = 0;
+    u32 cached_accept_state = 0;
 
-    DEBUG_PRINTF("accel %hu, accept %hu\n", accel_limit, accept_limit);
+    DEBUG_PRINTF("accel %hu, accept %u\n", m->accel_limit_8, accept_limit);
 
-    DEBUG_PRINTF("s: %hhu, len %zu\n", s, len);
+    DEBUG_PRINTF("s: %u, len %zu\n", s, len);
 
     const u8 *min_accel_offset = c;
     if (!m->has_accel || len < ACCEL_MIN_LEN) {
@@ -310,124 +405,119 @@ char mcclellanExec8_i(const struct mcclellan *m, u8 *state, const u8 *buf,
     goto with_accel;
 
 without_accel:
-    while (c < min_accel_offset && s) {
-        u8 cprime = m->remap[*(c++)];
-        DEBUG_PRINTF("c: %02hhx '%c' cp:%02hhx\n", *(c-1),
-                     ourisprint(*(c-1)) ? *(c-1) : '?', cprime);
-        s = succ_table[((u32)s << as) + cprime];
-        DEBUG_PRINTF("s: %hhu\n", s);
+    do {
+        assert(c < min_accel_offset);
+        if (!s) {
+            goto exit;
+        }
+
+        s = doNormal8(m, &c, min_accel_offset, s, 0, mode);
 
         if (mode != NO_MATCHES && s >= accept_limit) {
             if (mode == STOP_AT_MATCH) {
                 DEBUG_PRINTF("match - pausing\n");
                 *state = s;
                 *c_final = c - 1;
-                return MO_CONTINUE_MATCHING;
+                return MO_MATCHES_PENDING;
             }
 
             u64a loc = (c - 1) - buf + offAdj + 1;
             if (single) {
                 DEBUG_PRINTF("reporting %u\n", m->arb_report);
                 if (cb(0, loc, m->arb_report, ctxt) == MO_HALT_MATCHING) {
-                    return MO_HALT_MATCHING;
+                    return MO_DEAD;
                 }
             } else if (doComplexReport(cb, ctxt, m, s, loc, 0,
-                                       &cached_accept_state,
-                                       &cached_accept_id)
+                                       &cached_accept_state, &cached_accept_id)
                        == MO_HALT_MATCHING) {
-                return MO_HALT_MATCHING;
+                return MO_DEAD;
             }
         }
+
+        assert(c <= min_accel_offset);
+    } while (c < min_accel_offset);
+
+    if (c == c_end) {
+        goto exit;
     }
 
 with_accel:
-    while (c < c_end && s) {
-        u8 cprime = m->remap[*(c++)];
-        DEBUG_PRINTF("c: %02hhx '%c' cp:%02hhx\n", *(c-1),
-                     ourisprint(*(c-1)) ? *(c-1) : '?', cprime);
-        s = succ_table[((u32)s << as) + cprime];
-        DEBUG_PRINTF("s: %hhu\n", s);
+    do {
+        u32 accel_limit = m->accel_limit_8;
+        assert(c < c_end);
 
-        if (s >= accel_limit) { /* accept_limit >= accel_limit */
-            if (mode != NO_MATCHES && s >= accept_limit) {
-                if (mode == STOP_AT_MATCH) {
-                    DEBUG_PRINTF("match - pausing\n");
-                    *state = s;
-                    *c_final = c - 1;
-                    return MO_CONTINUE_MATCHING;
-                }
+        if (!s) {
+            goto exit;
+        }
 
-                u64a loc = (c - 1) - buf + offAdj + 1;
-                if (single) {
-                    DEBUG_PRINTF("reporting %u\n", m->arb_report);
-                    if (cb(0, loc, m->arb_report, ctxt) == MO_HALT_MATCHING) {
-                        return MO_HALT_MATCHING;
-                    }
-                } else if (doComplexReport(cb, ctxt, m, s, loc, 0,
-                                           &cached_accept_state,
-                                           &cached_accept_id)
-                           == MO_HALT_MATCHING) {
-                    return MO_HALT_MATCHING;
-                }
-            } else if (aux[s].accel_offset) {
-                DEBUG_PRINTF("skipping\n");
-
-                const union AccelAux *aaux = (const void *)((const char *)m
-                                                         + aux[s].accel_offset);
-                const u8 *c2 = run_accel(aaux, c, c_end);
-
-                if (c2 < min_accel_offset + BAD_ACCEL_DIST) {
-                    min_accel_offset = c2 + BIG_ACCEL_PENALTY;
-                } else {
-                    min_accel_offset = c2 + SMALL_ACCEL_PENALTY;
-                }
-
-                if (min_accel_offset >= c_end - ACCEL_MIN_LEN) {
-                    min_accel_offset = c_end;
-                }
-
-                DEBUG_PRINTF("advanced %zd, next accel chance in %zd/%zd\n",
-                         c2 - c, min_accel_offset - c2, c_end - c2);
-
-                c = c2;
+        if (s >= accel_limit && aux[s].accel_offset) {
+            c = run_mcclellan_accel(m, aux, s, &min_accel_offset, c, c_end);
+            if (c == c_end) {
+                goto exit;
+            } else {
                 goto without_accel;
             }
         }
-    }
+        s = doNormal8(m, &c, c_end, s, 1, mode);
 
+        if (mode != NO_MATCHES && s >= accept_limit) {
+            if (mode == STOP_AT_MATCH) {
+                DEBUG_PRINTF("match - pausing\n");
+                *state = s;
+                *c_final = c - 1;
+                return MO_MATCHES_PENDING;
+            }
+
+            u64a loc = (c - 1) - buf + offAdj + 1;
+            if (single) {
+                DEBUG_PRINTF("reporting %u\n", m->arb_report);
+                if (cb(0, loc, m->arb_report, ctxt) == MO_HALT_MATCHING) {
+                    return MO_DEAD;
+                }
+            } else if (doComplexReport(cb, ctxt, m, s, loc, 0,
+                                       &cached_accept_state, &cached_accept_id)
+                       == MO_HALT_MATCHING) {
+                return MO_DEAD;
+            }
+        }
+
+        assert(c <= c_end);
+    } while (c < c_end);
+
+exit:
     *state = s;
     if (mode == STOP_AT_MATCH) {
         *c_final = c_end;
     }
-    return MO_CONTINUE_MATCHING;
+    return MO_ALIVE;
 }
 
 static never_inline
-char mcclellanExec8_i_cb(const struct mcclellan *m, u8 *state, const u8 *buf,
+char mcclellanExec8_i_cb(const struct mcclellan *m, u32 *state, const u8 *buf,
                          size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                          char single, const u8 **final_point) {
     return mcclellanExec8_i(m, state, buf, len, offAdj, cb, ctxt, single,
-                            final_point, CALLBACK_OUTPUT);
+                          final_point, CALLBACK_OUTPUT);
 }
 
 static never_inline
-char mcclellanExec8_i_sam(const struct mcclellan *m, u8 *state, const u8 *buf,
+char mcclellanExec8_i_sam(const struct mcclellan *m, u32 *state, const u8 *buf,
                           size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                           char single, const u8 **final_point) {
     return mcclellanExec8_i(m, state, buf, len, offAdj, cb, ctxt, single,
-                            final_point, STOP_AT_MATCH);
+                          final_point, STOP_AT_MATCH);
 }
 
 static never_inline
-char mcclellanExec8_i_nm(const struct mcclellan *m, u8 *state, const u8 *buf,
+char mcclellanExec8_i_nm(const struct mcclellan *m, u32 *state, const u8 *buf,
                          size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                          char single, const u8 **final_point) {
     return mcclellanExec8_i(m, state, buf, len, offAdj, cb, ctxt, single,
-                            final_point, NO_MATCHES);
+                          final_point, NO_MATCHES);
 }
 
 static really_inline
-char mcclellanExec8_i_ni(const struct mcclellan *m, u8 *state, const u8 *buf,
+char mcclellanExec8_i_ni(const struct mcclellan *m, u32 *state, const u8 *buf,
                          size_t len, u64a offAdj, NfaCallback cb, void *ctxt,
                          char single, const u8 **final_point,
                          enum MatchMode mode) {
@@ -445,7 +535,7 @@ char mcclellanExec8_i_ni(const struct mcclellan *m, u8 *state, const u8 *buf,
 }
 
 static really_inline
-char mcclellanCheckEOD(const struct NFA *nfa, u16 s, u64a offset,
+char mcclellanCheckEOD(const struct NFA *nfa, u32 s, u64a offset,
                        NfaCallback cb, void *ctxt) {
     const struct mcclellan *m = getImplNfa(nfa);
     const struct mstate_aux *aux = get_aux(m, s);
@@ -466,7 +556,7 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
     s64a sp;
 
     assert(ISALIGNED_N(q->state, 2));
-    u16 s = *(u16 *)q->state;
+    u32 s = *(u16 *)q->state;
 
     if (q->report_current) {
         assert(s);
@@ -478,7 +568,7 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
             rv = cb(0, q_cur_offset(q), m->arb_report, context);
         } else {
             u32 cached_accept_id = 0;
-            u16 cached_accept_state = 0;
+            u32 cached_accept_state = 0;
 
             rv = doComplexReport(cb, context, m, s, q_cur_offset(q), 0,
                                  &cached_accept_state, &cached_accept_id);
@@ -487,7 +577,7 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         q->report_current = 0;
 
         if (rv == MO_HALT_MATCHING) {
-            return MO_HALT_MATCHING;
+            return MO_DEAD;
         }
     }
 
@@ -496,12 +586,6 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
     const u8 *cur_buf = sp < 0 ? hend : buffer;
 
-    char report = 1;
-    if (mode == CALLBACK_OUTPUT) {
-        /* we are starting inside the history buffer: matches are suppressed */
-        report = !(sp < 0);
-    }
-
     assert(q->cur);
     if (mode != NO_MATCHES && q->items[q->cur - 1].location > end) {
         DEBUG_PRINTF("this is as far as we go\n");
@@ -528,19 +612,20 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         /* do main buffer region */
         const u8 *final_look;
-        if (mcclellanExec16_i_ni(m, &s, cur_buf + sp, local_ep - sp,
-                                 offset + sp, cb, context, single, &final_look,
-                                 report ? mode : NO_MATCHES)
-            == MO_HALT_MATCHING) {
-            assert(report);
+        char rv = mcclellanExec16_i_ni(m, &s, cur_buf + sp, local_ep - sp,
+                                      offset + sp, cb, context, single,
+                                      &final_look, mode);
+        if (rv == MO_DEAD) {
             *(u16 *)q->state = 0;
-            return 0;
+            return MO_DEAD;
         }
-        if (mode == STOP_AT_MATCH && final_look != cur_buf + local_ep) {
+        if (mode == STOP_AT_MATCH && rv == MO_MATCHES_PENDING) {
             DEBUG_PRINTF("this is as far as we go\n");
+            DEBUG_PRINTF("state %u final_look %zd\n", s, final_look - cur_buf);
+
             assert(q->cur);
-            DEBUG_PRINTF("state %hu final_look %zd\n", s,
-                          final_look - cur_buf);
+            assert(final_look != cur_buf + local_ep);
+
             q->cur--;
             q->items[q->cur].type = MQE_START;
             q->items[q->cur].location = final_look - cur_buf + 1; /* due to
@@ -549,6 +634,7 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
             return MO_MATCHES_PENDING;
         }
 
+        assert(rv == MO_ALIVE);
         assert(q->cur);
         if (mode != NO_MATCHES && q->items[q->cur].location > end) {
             DEBUG_PRINTF("this is as far as we go\n");
@@ -563,7 +649,6 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         if (sp == 0) {
             cur_buf = buffer;
-            report = 1;
         }
 
         if (sp != ep) {
@@ -582,7 +667,7 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         case MQE_END:
             *(u16 *)q->state = s;
             q->cur++;
-            return s ? MO_ALIVE : 0;
+            return s ? MO_ALIVE : MO_DEAD;
         default:
             assert(!"invalid queue event");
         }
@@ -591,18 +676,18 @@ char nfaExecMcClellan16_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
     }
 }
 
-static really_inline really_flatten
-char nfaExecMcClellan16_Bi(const struct NFA *n, u64a offset,
-                           const u8 *buffer, size_t length,
-                           NfaCallback cb, void *context, char single) {
+static really_inline
+char nfaExecMcClellan16_Bi(const struct NFA *n, u64a offset, const u8 *buffer,
+                           size_t length, NfaCallback cb, void *context,
+                           char single) {
     assert(n->type == MCCLELLAN_NFA_16);
     const struct mcclellan *m = getImplNfa(n);
-    u16 s = m->start_anchored;
+    u32 s = m->start_anchored;
 
     if (mcclellanExec16_i(m, &s, buffer, length, offset, cb, context, single,
                           NULL, CALLBACK_OUTPUT)
-        == MO_HALT_MATCHING) {
-        return 0;
+        == MO_DEAD) {
+        return s ? MO_ALIVE : MO_DEAD;
     }
 
     const struct mstate_aux *aux = get_aux(m, s);
@@ -611,19 +696,19 @@ char nfaExecMcClellan16_Bi(const struct NFA *n, u64a offset,
         doComplexReport(cb, context, m, s, offset + length, 1, NULL, NULL);
     }
 
-    return !!s;
+    return MO_ALIVE;
 }
 
 static really_inline
 char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
-                          const u8 *hend, NfaCallback cb, void *context,
-                          struct mq *q, char single, s64a end,
-                          enum MatchMode mode) {
+                           const u8 *hend, NfaCallback cb, void *context,
+                           struct mq *q, char single, s64a end,
+                           enum MatchMode mode) {
     assert(n->type == MCCLELLAN_NFA_8);
     const struct mcclellan *m = getImplNfa(n);
     s64a sp;
 
-    u8 s = *(u8 *)q->state;
+    u32 s = *(u8 *)q->state;
 
     if (q->report_current) {
         assert(s);
@@ -635,7 +720,7 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
             rv = cb(0, q_cur_offset(q), m->arb_report, context);
         } else {
             u32 cached_accept_id = 0;
-            u16 cached_accept_state = 0;
+            u32 cached_accept_state = 0;
 
             rv = doComplexReport(cb, context, m, s, q_cur_offset(q), 0,
                                  &cached_accept_state, &cached_accept_id);
@@ -644,7 +729,7 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         q->report_current = 0;
 
         if (rv == MO_HALT_MATCHING) {
-            return MO_HALT_MATCHING;
+            return MO_DEAD;
         }
     }
 
@@ -653,12 +738,6 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
     const u8 *cur_buf = sp < 0 ? hend : buffer;
 
-    char report = 1;
-    if (mode == CALLBACK_OUTPUT) {
-        /* we are starting inside the history buffer: matches are suppressed */
-        report = !(sp < 0);
-    }
-
     if (mode != NO_MATCHES && q->items[q->cur - 1].location > end) {
         DEBUG_PRINTF("this is as far as we go\n");
         q->cur--;
@@ -686,17 +765,20 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         }
 
         const u8 *final_look;
-        if (mcclellanExec8_i_ni(m, &s, cur_buf + sp, local_ep - sp, offset + sp,
-                                cb, context, single, &final_look,
-                                report ? mode : NO_MATCHES)
-            == MO_HALT_MATCHING) {
+        char rv = mcclellanExec8_i_ni(m, &s, cur_buf + sp, local_ep - sp,
+                                     offset + sp, cb, context, single,
+                                     &final_look, mode);
+        if (rv == MO_HALT_MATCHING) {
             *(u8 *)q->state = 0;
-            return 0;
+            return MO_DEAD;
         }
-        if (mode == STOP_AT_MATCH && final_look != cur_buf + local_ep) {
-            /* found a match */
-            DEBUG_PRINTF("found a match\n");
+        if (mode == STOP_AT_MATCH && rv == MO_MATCHES_PENDING) {
+            DEBUG_PRINTF("this is as far as we go\n");
+            DEBUG_PRINTF("state %u final_look %zd\n", s, final_look - cur_buf);
+
             assert(q->cur);
+            assert(final_look != cur_buf + local_ep);
+
             q->cur--;
             q->items[q->cur].type = MQE_START;
             q->items[q->cur].location = final_look - cur_buf + 1; /* due to
@@ -705,6 +787,7 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
             return MO_MATCHES_PENDING;
         }
 
+        assert(rv == MO_ALIVE);
         assert(q->cur);
         if (mode != NO_MATCHES && q->items[q->cur].location > end) {
             DEBUG_PRINTF("this is as far as we go\n");
@@ -720,7 +803,6 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
 
         if (sp == 0) {
             cur_buf = buffer;
-            report = 1;
         }
 
         if (sp != ep) {
@@ -739,7 +821,7 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
         case MQE_END:
             *(u8 *)q->state = s;
             q->cur++;
-            return s ? MO_ALIVE : 0;
+            return s ? MO_ALIVE : MO_DEAD;
         default:
             assert(!"invalid queue event");
         }
@@ -748,18 +830,18 @@ char nfaExecMcClellan8_Q2i(const struct NFA *n, u64a offset, const u8 *buffer,
     }
 }
 
-static really_inline really_flatten
+static really_inline
 char nfaExecMcClellan8_Bi(const struct NFA *n, u64a offset, const u8 *buffer,
                           size_t length, NfaCallback cb, void *context,
                           char single) {
     assert(n->type == MCCLELLAN_NFA_8);
     const struct mcclellan *m = getImplNfa(n);
-    u8 s = (u8)m->start_anchored;
+    u32 s = m->start_anchored;
 
     if (mcclellanExec8_i(m, &s, buffer, length, offset, cb, context, single,
                          NULL, CALLBACK_OUTPUT)
-        == MO_HALT_MATCHING) {
-        return 0;
+        == MO_DEAD) {
+        return MO_DEAD;
     }
 
     const struct mstate_aux *aux = get_aux(m, s);
@@ -768,7 +850,7 @@ char nfaExecMcClellan8_Bi(const struct NFA *n, u64a offset, const u8 *buffer,
         doComplexReport(cb, context, m, s, offset + length, 1, NULL, NULL);
     }
 
-    return s;
+    return s ? MO_ALIVE : MO_DEAD;
 }
 
 char nfaExecMcClellan8_B(const struct NFA *n, u64a offset, const u8 *buffer,
@@ -827,7 +909,7 @@ char nfaExecMcClellan8_reportCurrent(const struct NFA *n, struct mq *q) {
     const struct mcclellan *m = getImplNfa(n);
     NfaCallback cb = q->cb;
     void *ctxt = q->context;
-    u8 s = *(u8 *)q->state;
+    u32 s = *(u8 *)q->state;
     u8 single = m->flags & MCCLELLAN_FLAG_SINGLE;
     u64a offset = q_cur_offset(q);
     assert(q_cur_type(q) == MQE_START);
@@ -839,7 +921,7 @@ char nfaExecMcClellan8_reportCurrent(const struct NFA *n, struct mq *q) {
             cb(0, offset, m->arb_report, ctxt);
         } else {
             u32 cached_accept_id = 0;
-            u16 cached_accept_state = 0;
+            u32 cached_accept_state = 0;
 
             doComplexReport(cb, ctxt, m, s, offset, 0, &cached_accept_state,
                             &cached_accept_id);
@@ -853,12 +935,12 @@ char nfaExecMcClellan16_reportCurrent(const struct NFA *n, struct mq *q) {
     const struct mcclellan *m = getImplNfa(n);
     NfaCallback cb = q->cb;
     void *ctxt = q->context;
-    u16 s = *(u16 *)q->state;
+    u32 s = *(u16 *)q->state;
     const struct mstate_aux *aux = get_aux(m, s);
     u8 single = m->flags & MCCLELLAN_FLAG_SINGLE;
     u64a offset = q_cur_offset(q);
     assert(q_cur_type(q) == MQE_START);
-    DEBUG_PRINTF("state %hu\n", s);
+    DEBUG_PRINTF("state %u\n", s);
     assert(s);
 
     if (aux->accept) {
@@ -867,7 +949,7 @@ char nfaExecMcClellan16_reportCurrent(const struct NFA *n, struct mq *q) {
             cb(0, offset, m->arb_report, ctxt);
         } else {
             u32 cached_accept_id = 0;
-            u16 cached_accept_state = 0;
+            u32 cached_accept_state = 0;
 
             doComplexReport(cb, ctxt, m, s, offset, 0, &cached_accept_state,
                             &cached_accept_id);
@@ -1041,7 +1123,7 @@ void nfaExecMcClellan8_SimpStream(const struct NFA *nfa, char *state,
                                   size_t len, NfaCallback cb, void *ctxt) {
     const struct mcclellan *m = getImplNfa(nfa);
 
-    u8 s = top ? m->start_anchored : *(u8 *)state;
+    u32 s = top ? m->start_anchored : *(u8 *)state;
 
     if (m->flags & MCCLELLAN_FLAG_SINGLE) {
         mcclellanExec8_i(m, &s, buf + start_off, len - start_off,
@@ -1059,14 +1141,14 @@ void nfaExecMcClellan16_SimpStream(const struct NFA *nfa, char *state,
                                    size_t len, NfaCallback cb, void *ctxt) {
     const struct mcclellan *m = getImplNfa(nfa);
 
-    u16 s = top ? m->start_anchored : unaligned_load_u16(state);
+    u32 s = top ? m->start_anchored : unaligned_load_u16(state);
 
     if (m->flags & MCCLELLAN_FLAG_SINGLE) {
         mcclellanExec16_i(m, &s, buf + start_off, len - start_off,
-                         start_off, cb, ctxt, 1, NULL, CALLBACK_OUTPUT);
+                          start_off, cb, ctxt, 1, NULL, CALLBACK_OUTPUT);
     } else {
         mcclellanExec16_i(m, &s, buf + start_off, len - start_off,
-                         start_off, cb, ctxt, 0, NULL, CALLBACK_OUTPUT);
+                          start_off, cb, ctxt, 0, NULL, CALLBACK_OUTPUT);
     }
 
     unaligned_store_u16(state, s);
@@ -1087,13 +1169,15 @@ char nfaExecMcClellan16_testEOD(const struct NFA *nfa, const char *state,
                              context);
 }
 
-char nfaExecMcClellan8_queueInitState(UNUSED const struct NFA *nfa, struct mq *q) {
+char nfaExecMcClellan8_queueInitState(UNUSED const struct NFA *nfa,
+                                      struct mq *q) {
     assert(nfa->scratchStateSize == 1);
     *(u8 *)q->state = 0;
     return 0;
 }
 
-char nfaExecMcClellan16_queueInitState(UNUSED const struct NFA *nfa, struct mq *q) {
+char nfaExecMcClellan16_queueInitState(UNUSED const struct NFA *nfa,
+                                       struct mq *q) {
     assert(nfa->scratchStateSize == 2);
     assert(ISALIGNED_N(q->state, 2));
     *(u16 *)q->state = 0;
