@@ -33,7 +33,10 @@
 #include "rdfa.h"
 #include "util/bytecode_ptr.h"
 #include "util/charreach.h"
-#include "util/ue2_containers.h"
+#include "util/flat_containers.h"
+
+#include <memory>
+#include <set>
 
 struct NFA;
 
