@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -249,15 +249,6 @@ string describeClass(const CharReach &cr, size_t maxLength,
     return oss.str();
 }
 
-string describeClasses(const std::vector<CharReach> &v, size_t maxClassLength,
-                       enum cc_output_t out_type) {
-    std::ostringstream oss;
-    for (const auto &cr : v) {
-        describeClass(oss, cr, maxClassLength, out_type);
-    }
-    return oss.str();
-}
-
 // C stdio wrapper
 void describeClass(FILE *f, const CharReach &cr, size_t maxLength,
                    enum cc_output_t out_type) {
