@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
 
 #include "config.h"
 
-#include "util/ue2_containers.h"
+#include "util/flat_containers.h"
 #include "ue2common.h"
 
 #include "gtest/gtest.h"
@@ -403,6 +403,11 @@ TEST(flat_map, max_size) {
     ASSERT_LE(1ULL << 24, f.max_size());
 }
 
+template<typename FlatMap>
+size_t hash_value(const FlatMap &f) {
+    return std::hash<FlatMap>()(f);
+}
+
 TEST(flat_map, hash_value) {
     const vector<pair<u32, u32>> input = {
         {0, 0}, {3, 1}, {76, 2}, {132, 3}, {77, 4}, {99999, 5}, {100, 6}};
